## mtmain.sh --- shell script for building Matlab MEX-files.

# Copyright (C) 1999--2004 Ralph Schleicher

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307, USA.

## Commentary:

# The following configuration variables are merged in by the GNU
# Autoconf command AX_PROG_MEXTOOL.  Do not edit until you known
# what you are doing.

#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#

## Code:

# Short program invocation name (without directory).
prog=`echo "$0" | sed -e 's,^.*/,,'`

# Version information.
version_info="\
mtmain.sh 2.0 (Mextool 2.1)

Copyright (C) 1999--2003 Ralph Schleicher

This program is free software and supplied as it is.  There is NO
WARRANTY; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE."

# Help text.
help_text="\
Usage: $prog ARGUMENT...

Create a Matlab MEX-file using GNU Libtool.  All non-source file names
are passed to the linker without being compiled.

Options:
  -ada FILE         Use FILE as the Ada package specification.
  -ARCH             Assume host architecture ARCH.
  -argcheck         Check arguments of Matlab API functions.
  -c                Compile only; do not link.
  -DNAME[=DEF]      Define preprocessor macro name NAME.
  -f FILE           Use options file FILE.
  -fortran          Specify that the gateway function is in Fortran.
  -g                Add debugging options.
  -h, -help         Print this help and exit successfully.
  -IDIR             Add DIR to the search path for include files.
  -inline           Enable inline code for matrix access functions.
  -LDIR             Add DIR to the search path for libraries.
  -lNAME            Link with object library NAME.
  -n, -dry-run      Do not execute any commands.
  -O                Add optimization options.
  -outdir DIR       Place all output files in directory DIR.
  -o, -output NAME  Create MEX-file NAME.
  -quiet            Do not print compile and link commands.
  -rpath DIR        The MEX-file will eventually be installed in DIR.
  -setup            Create a new default options file.
  -silent           Do not print compile and link commands.
  -trace            Print internal commands.
  -UNAME            Undefine preprocessor macro name NAME.
  -V4               Create a Matlab 4 compatible MEX-file.
  -v, -verbose      Be verbose and print all compile and link commands.
  -version          Print version number and exit successfully.
  VAR=VALUE         Override options file setting for variable VAR.

ARCH is either \`alpha', \`glnx86', \`hp700', \`hpux', \`ibm_rs', \`sgi',
\`sol2', or \`win32'.

Some features are still experimental (C++, Fortran), unsupported (Ada),
or silently ignored.  See the manual for more details.

Report bugs to <bug-mextool@nunatak.allgaeu.org>."

# Sed script for deleting a leading X.
X='s,^X,,'

# Sed script for quoting shell meta characters that are
# still active within double-quotes.
quote='s,\([\\`\\"\\$\\\\]\),\\\1,g'

# Sed script for deleting the directory part of a file name.
prefix='s,^.*/,,'

# Sed script for deleting the file name extension.
suffix='s,\.[^.]*$,,'

# Single options.
arch=$MATLAB_ARCH
check_arg=no
compile_only=no
options_file=
gateway=none
debug=no
help=no
inline=no
dry_run=no
optimize=no
outdir=
output=
rpath=$matlabarchdir
setup=no
trace=:
compat=no
verbose=no
version=no

# Accumulated options.
cppflags=
ldadd=
vars=

# Source file arguments.
sources=

# Input file name containing the gateway function.
input=

# Process command line arguments.
prev=

for arg
do
    # If the previous option needs an argument, assign it.
    if test -n "$prev"
    then
	case $arg in
	  -*)
	    echo "$prog: Option \`$prev' requires an argument" >&2
	    echo "Try \`$0 --help' for more information." >&2
	    exit 1
	    ;;
	esac
	case $prev in
	  -ada)
	    case $gateway in
	      none | ada)
		gateway=ada
		;;
	      *)
		echo "$prog: Only one type of gateway function is allowed" >&2
		echo "Try \`$0 --help' for more information." >&2
		exit 1
		;;
	    esac
	    case $arg in
	      *.ads)
		# Ada package specification.
		input=$arg
		;;
	      *)
		echo "$prog:$arg: Invalid Ada package specification file name" >&2
		echo "Try \`$0 --help' for more information." >&2
		exit 1
		;;
	    esac
	    ;;
	  -f)
	    options_file=$arg
	    ;;
	  -outdir)
	    if test -n "$arg"
	    then
		case $arg in
		  */)
		    ;;
		  *)
		    arg="$arg/"
		    ;;
		esac
	    fi
	    outdir=$arg
	    ;;
	  -output)
	    case $arg in
	      */*)
		# Ignore value of $outdir for absolute or explicit
		# relative output file names.
		output="$arg.la"
		;;
	      *)
		output="\${outdir}$arg.la"
		;;
	    esac
	    ;;
	  -rpath)
	    rpath=$arg
	    ;;
	esac

	prev=
	continue
    fi

    case $arg in
      -ada | --ada | --ad)
	prev=-ada
	;;
      -alpha | --alpha | --alph | --alp | --al)
	arch=alpha
	;;
      -glnx86 | --glnx86 | --glnx8 | --glnx | --gln | --gl | --g)
	arch=glnx86
	;;
      -hp700 | --hp700 | --hp70 | --hp7)
	arch=hp700
	;;
      -hpux | --hpux | --hpu)
	arch=hpux
	;;
      -ibm_rs | --ibm_rs | --ibm_r | --ibm_ | --ibm | --ib)
	arch=ibm_rs
	;;
      -ibm-rs | --ibm-rs | --ibm-r | --ibm-)
	arch=ibm_rs
	;;
      -sgi | --sgi | --sg)
	arch=sgi
	;;
      -sol2 | --sol2 | --sol | --so)
	arch=sol2
	;;
      -win32 | --win32 | --win3 | --win | --wi | --w)
	arch=win32
	;;
      -argcheck | --argcheck | --argchec | --argche | --argch | --argc | --arg | --ar)
	check_arg=yes
	;;
      -c)
	compile_only=yes
	;;
      -f)
	prev=$arg
	;;
      -fortran | --fortran | --fortra | --fortr | --fort | --for | --fo | --f)
	case $gateway in
	  none | fortran)
	    gateway=fortran
	    ;;
	  *)
	    echo "$prog: Only one type of gateway function is allowed" >&2
	    echo "Try \`$0 --help' for more information." >&2
	    exit 1
	    ;;
	esac
	;;
      -g)
	debug=yes
	;;
      -h | -help | --help | --hel | --he | --h)
	help=yes
	;;
      -inline | --inline | --inlin | --inli | --inl | --in)
	inline=yes
	;;
      -n | -dry-run | --dry-run | --dry-ru | --dry-r | --dry- | --dry | --dr | --d)
	dry_run=yes
	;;
      -O)
	optimize=yes
	;;
      -outdir | --outdir | --outdi | --outd)
	prev=-outdir
	;;
      -o | -output | --output | --outpu | --outp)
	prev=-output
	;;
      -quiet | --quiet | --quie | --qui | --qu | --q)
	verbose=no
	;;
      -rpath | --rpath | --rpat | --rpa | --rp | --r)
	prev=-rpath
	;;
      -setup | --setup | --setu | --set | --se)
	setup=yes
	;;
      -silent | --silent | --silen | --sile | --sil | --si)
	verbose=no
	;;
      -trace | --trace | --trac | --tra | --tr | --t)
	trace=echo
	;;
      -V4 | --V4 | --V)
	compat=yes
	;;
      -v | -verbose | --verbose | --verbos | --verbo | --verb)
	verbose=yes
	;;
      -version | --version | --versio | --versi | --vers)
	version=yes
	;;
      -D* | -I* | -U*)
	case $arg in
	  -DARGCHECK | -DARGCHECK=* | -UARGCHECK)
	    echo "$prog: warning: Providing \`$arg' may conflict with \`-argcheck'" >&2
	    ;;
	  -DARRAY_ACCESS_INLINING | -DARRAY_ACCESS_INLINING=* | -UARRAY_ACCESS_INLINING)
	    echo "$prog: warning: Providing \`$arg' may conflict with \`-inline'" >&2
	    ;;
	  -DV4_COMPAT | -DV4_COMPAT=* | -UV4_COMPAT)
	    echo "$prog: warning: Providing \`$arg' may conflict with \`-V4'" >&2
	    ;;
	  -DMATLAB_MEX_FILE | -DMATLAB_MEX_FILE=* | -UMATLAB_MEX_FILE)
	    echo "$prog: warning: Providing \`$arg' is in general a bad idea" >&2
	    ;;
	esac
	arg=`echo "X$arg" | sed -e "$X" -e "$quote"`
	case $arg in
	  *[\	\ \#\&\'\(\)\*\;\<\>\?\^\{\|\}\~]* | *[* | *]* | "")
	    arg="\"$arg\""
	    ;;
	esac
	if test -z "$cppflags"
	then
	    cppflags="$arg"
	else
	    cppflags="$cppflags $arg"
	fi
	;;
      -L* | -l*)
	arg=`echo "X$arg" | sed -e "$X" -e "$quote"`
	case $arg in
	  *[\	\ \#\&\'\(\)\*\;\<\>\?\^\{\|\}\~]* | *[* | *]* | "")
	    arg="\"$arg\""
	    ;;
	esac
	if test -z "$ldadd"
	then
	    ldadd="$arg"
	else
	    ldadd="$ldadd $arg"
	fi
	;;
      -*)
	echo "$prog: Unrecognized option \`$arg'" >&2
	echo "Try \`$0 --help' for more information." >&2
	exit 1
	;;
      MATLAB=* | CPPFLAGS=* | CC=* | CFLAGS=* | CXX=* | CXXFLAGS=* | F77=* | FFLAGS=* | LDFLAGS=* | LDADD=*)
	var=`echo "$arg" | sed -e 's,=.*,,'`
	val=`echo "$arg" | sed -e 's,[^=]*=,,' -e "$quote"`
	case $val in
	  *[\	\ \#\&\'\(\)\*\;\<\>\?\^\{\|\}\~]* | *[* | *]* | "")
	    val="\"$val\""
	    ;;
	esac
	arg="$var=$val"
	if test -z "$vars"
	then
	    vars="$arg"
	else
	    vars="$vars ; $arg"
	fi
	;;
      *.c | *.C | *.cc | *.cpp | *.cxx | *.f | *.F | *.for | *.FOR)
	# We generally belive that source files names are clean.
	in=$arg
	if test -z "$sources"
	then
	    sources="$in"
	else
	    sources="$sources $in"
	fi
	out=`echo "$arg" | sed -e "$prefix" -e "$suffix"`
	if test -z "$ldadd"
	then
	    ldadd="\${outdir}$out.lo"
	else
	    ldadd="$ldadd \${outdir}$out.lo"
	fi
	# Assume that the first source file contains the gateway function.
	case $gateway in
	  none)
	    case $arg in
	      *.c | *.C | *.cc | *.cpp | *.cxx)
		gateway=c
		input=$arg
		;;
	      *.f | *.F | *.for | *.FOR)
		gateway=fortran
		input=$arg
		;;
	    esac
	    ;;
	  fortran)
	    case $arg in
	      *.f | *.F | *.for | *.FOR)
		test -z "$input" && input=$arg
		;;
	    esac
	    ;;
	esac
	;;
      *.o | *.lo | *.a | *.la)
	in=$arg
	if test -z "$ldadd"
	then
	    ldadd="$in"
	else
	    ldadd="$ldadd $in"
	fi
	# Assume that the first object file contains the gateway function.
	case $arg in
	  *.o | *.lo)
	    test -z "$input" && input=$arg
	    ;;
	esac
	;;
      *)
	echo "$prog: Unrecognized argument \`$arg'" >&2
	echo "Try \`$0 --help' for more information." >&2
	exit 1
	;;
    esac
done

if test "$prev"
then
    echo "$prog: Option \`$prev' requires an argument" >&2
    echo "Try \`$0 --help' for more information." >&2
    exit 1
fi

if test $version = yes || test $help = yes
then
    if test $version = yes
    then
	echo "$version_info"
    fi
    if test $help = yes
    then
	if test $version = yes
	then
	    echo
	fi
	echo "$help_text"
    fi
    exit 0
fi

# Filter out unsupported features.
case $gateway in
  ada)
    echo "$prog: Ada gateway functions are not supported" >&2
    echo "Try \`$0 --help' for more information." >&2
    exit 1
    ;;
esac

if test setup = yes
then
    echo "$prog: Creating a new default options file is not supported" >&2
    echo "Try \`$0 --help' for more information." >&2
    exit 1
fi

if test $check_arg = yes && test $inline = yes
then
    echo "$prog: Options \`-argcheck' and \`-inline' are mutually exclusive" >&2
    echo "Try \`$0 --help' for more information." >&2
    exit 1
fi

# Finalize options processing.
case $arch in
  alpha)
    mexext=mexaxp
    ;;
  glnx86)
    mexext=mexglx
    ;;
  hp700)
    mexext=mexhp7
    ;;
  hpux)
    mexext=mexhpux
    ;;
  ibm_rs)
    mexext=mexrs6
    ;;
  sgi)
    mexext=mexsg
    ;;
  sol2)
    mexext=mexsol
    ;;
  win32)
    mexext=dll
    ;;
  *)
    echo "$prog: Invalid Matlab machine architecture \`$arch'" >&2
    echo "Try \`$0 --help' for more information." >&2
    exit 1
    ;;
esac

if test -z "$input"
then
    echo "$prog: Can not guess gateway function input file" >&2
    echo "Try \`$0 --help' for more information." >&2
    exit 1
fi

if test -z "$output"
then
    # Derive the output file name from the input file name that
    # contains the gateway function.
    out=`echo "$input" | sed -e "$prefix" -e "$suffix"`
    output="\${outdir}$out.la"
fi

# Shell command prefix to display commands.
if test $verbose = no
then
    show=:
else
    show=echo
fi

# Shell command prefix to execute commands.
if test $dry_run = yes
then
    run=:
else
    run=
fi

# Evaluate environment variables given on the command line.
if test -n "$vars"
then
    $trace "$vars"
    eval "$vars"
fi

# Check for GNU compilers.
$F77 --version 2> /dev/null | head -1 | grep '\(GNU Fortran\|g77\)' > /dev/null
if test $? -eq 0
then
    G77=yes
else
    G77=no
fi

# Collect all preprocessor options.
xcppflags="-DMATLAB_MEX_FILE"
if test $compat = yes
then
    xcppflags="$xcppflags -DV4_COMPAT"
fi
if test $check_arg = yes
then
    case $gateway in
      c)
	if test $compat = yes
	then
	    echo "$prog: warning: Option \`-argcheck' has no effect" >&2
	else
	    xcppflags="$xcppflags -DARGCHECK"
	    arg="$MATLAB/extern/src/mwdebug.c"
	    in=`echo "$arg" | sed -e "$quote"`
	    case $in in
	      *[\	\ \#\&\'\(\)\*\;\<\>\?\^\{\|\}\~]* | *[* | *]*)
	        in="\"$in\""
		;;
	    esac
	    if test -z "$sources"
	    then
		sources="$in"
	    else
		sources="$in $sources"
	    fi
	    out=`echo "$arg" | sed -e "$prefix" -e "$suffix"`
	    if test -z "$ldadd"
	    then
		ldadd="\${outdir}$out.lo"
	    else
		ldadd="\${outdir}$out.lo $ldadd"
	    fi
	fi
	;;
      *)
	echo "$prog: warning: Option \`-argcheck' has no effect" >&2
	;;
    esac
fi
if test $inline = yes
then
    if test $compat = yes
    then
	echo "$prog: warning: Option \`-inline' has no effect" >&2
    else
	xcppflags="$xcppflags -DARRAY_ACCESS_INLINING"
    fi
fi
arg=`echo "X$MATLAB" | sed -e "$X" -e "$quote"`
case $arg in
  *[\	\ \#\&\'\(\)\*\;\<\>\?\^\{\|\}\~]* | *[* | *]*)
    xcppflags="$xcppflags \"-I$arg/extern/include\""
    xcppflags="$xcppflags \"-I$arg/simulink/include\""
    xcppflags="$xcppflags \"-I$arg/rtw/c/src\""
    xcppflags="$xcppflags \"-I$arg/rtw/c/libsrc\""
    ;;
  *)
    xcppflags="$xcppflags -I$arg/extern/include"
    xcppflags="$xcppflags -I$arg/simulink/include"
    xcppflags="$xcppflags -I$arg/rtw/c/src"
    xcppflags="$xcppflags -I$arg/rtw/c/libsrc"
    ;;
esac
if test -n "$CPPFLAGS"
then
    arg=`echo "X$CPPFLAGS" | sed -e "$X" -e "$quote"`
    xcppflags="$xcppflags $arg"
fi
if test -n "$cppflags"
then
    xcppflags="$xcppflags $cppflags"
fi

# How to run shell scripts.
xshell=`echo "X$SHELL" | sed -e "$X" -e "$quote"`

# Libtool command.
xlibtool=`echo "X$libtool" | sed -e "$X" -e "$quote"`
if test $verbose = no
then
    xlibtool="$xlibtool --quiet"
fi
if test $dry_run = yes
then
    xlibtool="$xlibtool --dry-run"
fi

# C compile command.
xcc=`echo "X$CC" | sed -e "$X" -e "$quote"`
xcflags=`echo "X$CFLAGS" | sed -e "$X" -e "$quote"`

cc="$xshell $xlibtool --mode=compile $xcc"
test -n "$xcppflags" && cc="$cc $xcppflags"
test -n "$xcflags" && cc="$cc $xcflags"

# C++ compile command.
xcxx=`echo "X$CXX" | sed -e "$X" -e "$quote"`
xcxxflags=`echo "X$CXXFLAGS" | sed -e "$X" -e "$quote"`

cxx="$xshell $xlibtool --mode=compile --tag=CXX $xcxx"
test -n "$xcppflags" && cxx="$cxx $xcppflags"
test -n "$xcxxflags" && cxx="$cxx $xcxxflags"

# F77 compile command.
xf77=`echo "X$F77" | sed -e "$X" -e "$quote"`
xfflags=`echo "X$FFLAGS" | sed -e "$X" -e "$quote"`

f77="$xshell $xlibtool --mode=compile --tag=F77 $xf77"
test -n "$xcppflags" && f77="$f77 $xcppflags"
test -n "$xfflags" && f77="$f77 $xfflags"
test "$G77" = yes && f77="$f77 -fno-underscoring"

# Create output directory.
if test -n "$outdir" && test ! -d "$outdir"
then
    command="mkdir $outdir || exit 1"
    $show "$command"
    $run eval "$command"
fi

# Compile all source files.
have_cxx=no
have_f77=no

for arg in $sources
do
    eval "in=$arg"
    case $in in
      *.c)
	command="$cc"
	;;
      *.C | *.c*)
	command="$cxx"
	have_cxx=yes
	;;
      *.f* | *.F*)
	command="$f77"
	have_f77=yes
	;;
    esac
    out=`echo "$in" | sed -e "$prefix" -e "$suffix" -e "$quote"`
    command="$command -c -o ${outdir}$out.lo $arg || exit 1"
    $trace "$command"
    eval "$command"
done

if test $compile_only = yes
then
    exit 0
fi

# Link command.
xldflags=`echo "X$LDFLAGS" | sed -e "$X" -e "$quote"`
xldadd=`echo "X$LDADD" | sed -e "$X" -e "$quote"`
xlibmtmex=`echo "X$libmtmex" | sed -e "$X" -e "$quote"`

case $have_cxx:$have_f77 in
  yes:*)
    ld="$xshell $xlibtool --mode=link --tag=CXX $xcxx"
    test -n "$xcppflags" && ld="$ld $xcppflags"
    test -n "$xcflags" && ld="$ld $xcxxflags"
    ;;
  *:yes)
    ld="$xshell $xlibtool --mode=link --tag=F77 $xf77"
    test -n "$xcppflags" && ld="$ld $xcppflags"
    test -n "$xfflags" && ld="$ld $xfflags"
    ;;
  *)
    ld="$xshell $xlibtool --mode=link $xcc"
    test -n "$xcppflags" && ld="$ld $xcppflags"
    test -n "$xcflags" && ld="$ld $xcflags"
    ;;
esac
test -n "$xldflags" && ld="$ld $xldflags"
ld="$ld -no-undefined -avoid-version -module"
test -n "$rpath" && ld="$ld -rpath $rpath"
test -n "$exports" && ld="$ld -export-symbols $exports"
ld="$ld -shrext .$mexext -o $output"
test -n "$ldadd" && ld="$ld $ldadd"
test -n "$xldadd" && ld="$ld $xldadd"
test -n "$xlibmtmex" && ld="$ld $xlibmtmex"

# Link the MEX-file.
command="$ld || exit 1"
$trace "$command"
eval "$command"

if test $dry_run = no
then
    xlibtool=`echo "X$libtool" | sed -e "$X" -e "$quote"`

    command="eval \`$xshell $xlibtool --config | egrep -e '^objdir=' | head -1\`"
    $trace "$command"
    eval "$command"

    command="eval \`egrep -e '^dlname=' $output | head -1\`"
    $trace "$command"
    eval "$command"

    command="rm -f $outdir$dlname && ln -s $objdir/$dlname $outdir$dlname"
    $show "$command"
    $run eval "$command"
fi

exit 0

# local variables:
# mode: shell-script
# sh-indentation: 4
# end:

## mtmain.sh ends here
